// DEFINE PINS
#define deviceResetPin 5
#define dataLED 2
#define LED1 13
#define LED2 12
#define LED3 8
#define LED4 7
#define ANALOGOUT 3
#define SENSOR1 0
#define SENSOR2 1

// Defines for the  program's status (used for staus variable): 
#define disconnected 0 
#define connecting 1 
#define connected 2 
#define requesting 3 
#define reading 4 
#define requestComplete 5 


int inByte= -1; 
char inString[32];       // string for incoming serial data 
int stringLen = 0;       // string index counter 
int status = 0;          // Lantronix device's connection status 
int secs = 0;            // second counter (used to sleep between checks)

int iReadCounter = 0;
long iAnalogIn1 = 0;
long iAnalogVal1 = 0;
long iAnalogPrev1a = 0;
long iAnalogPrev1b = 0;
long iAnalogMin1 = 10;
long iAnalogMax1 = 30;
long iAnalogIn2 = 0;
long iAnalogVal2 = 0;
long iAnalogPrev2a = 0;
long iAnalogPrev2b = 0;
long iAnalogMin2 = 0;
long iAnalogMax2 = 1000;

//Analog OUT stuff
int iAnalogVal = 255; //reading from server

long lastCompletionTime = millis(); 

void setup() {
  pinMode(dataLED, OUTPUT); 
  pinMode(LED1, OUTPUT); 
  pinMode(LED2, OUTPUT); 
  pinMode(LED3, OUTPUT); 
  pinMode(LED4, OUTPUT); 
  pinMode(deviceResetPin, OUTPUT); 
  pinMode(ANALOGOUT, OUTPUT);
  // start serial port, 9600 8-N-1: 
  Serial.begin(9600); 
  blink(1); 
  //reset Lantronix device: 
  resetDevice(); 
  // blink reset LED: 
  blink(1); 
}

// Take the Lantronix device's reset pin low to reset it: 
void resetDevice() {
  digitalWrite(deviceResetPin, LOW); 
  delay(50); 
  digitalWrite(deviceResetPin, HIGH); 
  // pause to let Lantronix device boot up: 
  delay(2000); 
}

// Blink the reset LED: 
void blink(int howManyTimes) { 
  int i; 
  for (i=0; i< howManyTimes; i++) { 
    digitalWrite(dataLED, HIGH); 
    delay(200); 
    digitalWrite(dataLED, LOW); 
    delay(200); 
  } 
} 

void loop () {
  stateCheck(); 
  updateReadings();
} //end main loop

void stateCheck() { 
  switch (status) { 
  case disconnected: 
    // attempt to connect to the server: 
    deviceConnect(); 
    break; 
  case connecting: 
    // until you get a C, keep trying to connect: 
    // read the serial port: 
    if (Serial.available()) { 
      inByte = Serial.read(); 
      if (inByte == 'C') {  // 'C' in ascii
      status = connected; 
      } 
      else { 
        // if you got anything other than a C, try again: 
        deviceConnect(); 
      } 
    } 
    break; 
  case connected: 
    // send HTTP GET request for CGI script: 
    httpRequest(); 
    break; 
  case requesting: 
    lookForData(); 
    break; 
  case reading: 
    readData(); 
    break; 
  case requestComplete: 
    waitForNextRequest(); 
  } 
}

void deviceConnect() { 
  //   send out the server address and 
  //   wait for a "C" byte to come back. 
  //   fill in your server's numerical address below: 
  Serial.print("C128.122.253.189/80\n"); 
  status = connecting; 
} 

void httpRequest() { 
  // make sure you've cleared the last byte 
  // from the last request: 
  inByte = -1; 
  // reset the string position counter: 
  stringLen = 0;
  //  Make HTTP GET request. Fill in the path to your version 
  //  of the CGI script: 
  Serial.print("GET /~am2678/NetObj/mh.php?caller=A&AnalogIn1=");
  Serial.print(iAnalogIn1, DEC);
  Serial.print("&AnalogIn2=");
  Serial.print(iAnalogIn2, DEC);
  Serial.print("&Sensor1=");
  Serial.print(iAnalogVal1, DEC);
  Serial.print("&Sensor2=");
  Serial.print(iAnalogVal2, DEC);
  Serial.print(" HTTP/1.1\n"); 
  //  Fill in your server's name: 
  Serial.print("HOST:itp.nyu.edu\n\n"); 
  // update the state of the program: 
  status = requesting; 
} 

void lookForData() { 
  // wait for bytes from server: 
  if (Serial.available()) { 
    inByte = Serial.read(); 
    // If you get a "<", what follows is the device info string. 
    // You need to read what follows the A. 
    if (inByte == '<') { 
      stringLen = 0; 
      status = reading; 
    } 
  } 
} 

void readData() { 
  if (Serial.available()) { 
    inByte = Serial.read();
    
    // Keep reading until you get a "<": 
    if (inByte != '>') { 
        //append server info into character array
        inString[stringLen] = inByte; 
        stringLen++; 
    } 
    // if you get a ">", you've reached the end
    else { 
      //pad string
      inString[stringLen + 1] = ' '; 
      inString[stringLen + 2] = ' ';
      inString[stringLen + 3] = ' '; 
      inString[stringLen + 4] = ' ';  
      interpretResults(); 
    } 
  } 
} 

void interpretResults() { 

  // Get Value of Devices
  int iLED1 = iGetServerVal(1);
  int iLED2 = iGetServerVal(2);
  int iLED3 = iGetServerVal(3);
  int iLED4 = iGetServerVal(4);
  
  if (iLED1 == 1) {
    digitalWrite(LED1, HIGH);
  } else {
    digitalWrite(LED1, LOW);
  }
  if (iLED2 == 1) {
    digitalWrite(LED2, HIGH);
  } else {
    digitalWrite(LED2, LOW);
  }
  if (iLED3 == 1) {
    digitalWrite(LED3, HIGH);
  } else {
    digitalWrite(LED3, LOW);
  }
  if (iLED4 == 1) {
    digitalWrite(LED4, HIGH);
  } else {
    digitalWrite(LED4, LOW);
  }
  
  //Analog Device
  setAnalogDevice(iGetServerVal(8));
  
  status = requestComplete; 
} 

void waitForNextRequest() { 
  //if (millis() - lastCompletionTime >= 10000) { 
    blink(1);
    // reset Lantronix device before next request: 
    resetDevice(); 
    status = disconnected; 
  //} 
} 

long stringToNumber(char thisString[], int length) { 
  int thisChar = 0; 
  long value = 0;
    // count from the leas significant digit to the most significant, 
  // multiply it by the appropriate power of ten, and add it to the whole: 
  for (thisChar = length-1; thisChar >=0; thisChar--) { 
    char thisByte = thisString[thisChar] - 48; 
    value = value + powerOfTen(thisByte, (length-1)- thisChar); 
  } 
  return value; 
} 
/* 
  This method takes a number between 0 and 9, 
 and multiplies it by ten raised to a second number. 
 */ 
long powerOfTen(char digit, int power) { 
  long val = 1; 
  if (power == 0) { 
    return digit; 
  } 
  else { 
    for (int i = power; i >=1 ; i--) { 
      val = 10 * val; 
    } 
    return digit * val; 
  } 
} 

int iGetServerVal(int iDeviceID) {
  int iReturn = 0;
  int iCurrentDevice = 0;
  char sDevice[5]; 
  char sValue[5]; 
  for (int i=0; i < stringLen; i++){ 
      
    // one digit ID
    if (inString[i] == '-' && inString[i + 2] == '_') {
      sDevice[0] = inString[i + 1];
      iCurrentDevice = stringToNumber(sDevice, 1);
    }
    // two digit ID
    if (inString[i] == '-' && inString[i + 3] == '_') {
      sDevice[0] = inString[i + 1];
      sDevice[1] = inString[i + 2];
      iCurrentDevice = stringToNumber(sDevice, 2);
    }
    
    // if current device, and no reading yet, get reading
    if (iCurrentDevice == iDeviceID && iReturn == 0) {
      // one digit reading
      if (inString[i] == '_' && inString[i + 2] == '-') {
        sValue[0] = inString[i + 1];
        iReturn = stringToNumber(sValue, 1);
      }
      // two digit reading
      if (inString[i] == '_' && inString[i + 3] == '-') {
        sValue[0] = inString[i + 1];
        sValue[1] = inString[i + 2];
        iReturn = stringToNumber(sValue, 2);
      }
      // three digit reading
      if (inString[i] == '_' && inString[i + 4] == '-') {
        sValue[0] = inString[i + 1];
        sValue[1] = inString[i + 2];
        sValue[2] = inString[i + 3];
        iReturn = stringToNumber(sValue, 3);
      }
      
    } //end if current device is found
    
  } //end for loop
 
  return iReturn; 
}

void updateReadings() {
  
  if (iReadCounter == 1) {
    // Read & smooth AnalogIn1 Value
    iAnalogVal1 = (analogRead(SENSOR1) + iAnalogPrev1a + iAnalogPrev1b) / 3;
    if (iAnalogVal1 > iAnalogMax1) { iAnalogVal1 = iAnalogMax1; }
    if (iAnalogVal1 < iAnalogMin1) { iAnalogVal1 = iAnalogMin1; }
    iAnalogPrev1b = iAnalogPrev1a;
    iAnalogPrev1a = iAnalogVal1;
    iAnalogIn1 = (255 * ((100 * (iAnalogVal1 - iAnalogMin1)) / (iAnalogMax1 - iAnalogMin1))) / 100;
  } 
  if (iReadCounter == 7) {
    // Read & smooth AnalogIn2 Value
    iAnalogVal2 = (analogRead(SENSOR2) + iAnalogPrev2a + iAnalogPrev2b) / 3;
    if (iAnalogVal2 > iAnalogMax2) { iAnalogVal2 = iAnalogMax2; }
    if (iAnalogVal2 < iAnalogMin2) { iAnalogVal2 = iAnalogMin2; }
    iAnalogPrev2b = iAnalogPrev2a;
    iAnalogPrev2a = iAnalogVal2;
    iAnalogIn2 = (255 * ((100 * (iAnalogVal2 - iAnalogMin2)) / (iAnalogMax2 - iAnalogMin2))) / 100;
  }
  iReadCounter++;
  if (iReadCounter == 14) {
    iReadCounter = 0;
  }
  
} // end updateReadings

void setAnalogDevice(int iAnalogVal) {
  //iAnalogVal should be 0 or 255 (from server) 
 if (iAnalogVal == 2) { digitalWrite(ANALOGOUT,0); }
 if (iAnalogVal == 255) { digitalWrite(ANALOGOUT,1); }
  
  //translate based on device range
  //server percentage = iAnalogVal/255
  //arduino value = serverpct * range + min
  //iAnalog = ((iAnalogVal/255) * (iMaxAnalog - iMinAnalog)) + iMinAnalog
  //pulse = ( ( (iAnalogVal*maxPulse) - (iAnalogVal*minPulse)  ) / 255) + minPulse;
  //pulse = (iAnalogVal / 10) * 15 + 500; 
  //analogWrite(9,iAnalogVal);
  
} // end setAnalogDevice
